from django.shortcuts import render, redirect
from django.core.urlresolvers import reverse
from django.http import HttpResponse, JsonResponse
from django.contrib.auth.decorators import login_required
from django.views.decorators.csrf import csrf_exempt
from rest_framework.authentication import SessionAuthentication, TokenAuthentication
from rest_framework.permissions import AllowAny, IsAuthenticated
from rest_framework.decorators import authentication_classes, permission_classes, api_view
from ohm2_handlers_light.parsers import get_as_or_get_default
from . import dispatcher



@api_view(['POST'])
@permission_classes((AllowAny,))
def signup(request):
	"""
	User signup (without authentication token)

	
	__Inputs__:

		- username (string, required): user's username
		- password (string, required): user's password
		- email (string-email, optional): user's email

	__Output__:

		- error (json-dict): describes the error (if known) with a code (integer) and a message (string).
		- ret (boolean): true if the signup completed succesfully.

	__Notes__:

		- If username is an email, the email parameter will be overriten by username.	
	
	"""

	keys = (
		("username", "username", ""),
		("password", "password", ""),
		("email", "email", ""),
	)
	res, error = dispatcher.signup(request, get_as_or_get_default(request.data, keys))
	if error:
		return JsonResponse({"error": error.regroup()})
	return JsonResponse(res)


@api_view(['POST'])
@permission_classes((AllowAny,))
def login(request):
	"""
	User login (without authentication token)
	

	__Inputs__:

		- username (string, required): user's username
		- password (string, required): user's password

	__Output__:

		- error (json-dict): describes the error (if known) with a code (integer) and a message (string).
		- ret (boolean): true if the login completed succesfully.

	__Notes__:

		- None

	"""
	keys = (
		("username", "username", ""),
		("password", "password", ""),
	)
	res, error = dispatcher.login(request, get_as_or_get_default(request.data, keys))
	if error:
		return JsonResponse({"error": error.regroup()})
	return JsonResponse(res)


@api_view(['POST'])
@authentication_classes((TokenAuthentication, SessionAuthentication))
@permission_classes((IsAuthenticated,))
def logout(request):
	"""
	User logout


	__Inputs__:

		- None

	__Output__:

		- error (json-dict): describes the error (if known) with a code (integer) and a message (string).
		- ret (boolean): true if the logout completed succesfully.

	__Notes__:

		- None

	"""
	keys = (
	)
	res, error = dispatcher.logout(request, get_as_or_get_default(request.data, keys))
	if error:
		return JsonResponse({"error": error.regroup()})
	return JsonResponse(res)



@api_view(['POST'])
@permission_classes((AllowAny,))
def signup_and_get_token(request):
	"""
	User signup (with authentication token)


	__Inputs__:
		- username (string, required): user's username
		- password (string, required): user's password
		- email (string-email, optional): user's email

	__Output__:
		- error (json-dict): describes the error (if known) with a code (integer) and a message (string).
		- ret (json-dict): 
			- token (string): user's authentication token

	__Notes__:
		- If username is an email, the email parameter will be overriten by username.

	"""
	keys = (
		("username", "username", ""),
		("password", "password", ""),
		("email", "email", ""),
	)
	res, error = dispatcher.signup(request, get_as_or_get_default(request.data, keys))
	if error:
		return JsonResponse({"error": error.regroup()})

	elif res.get("error") or not res.get("ret"):
		return JsonResponse({"error" : res.get("error")})

	else:
		res, error = dispatcher.create_authtoken(request, get_as_or_get_default(request.data, (("username", "username", ""),)))

			
	if error:
		return JsonResponse({"error": {"code": -1, "message" : "an error occured"}})
	return JsonResponse(res)


@api_view(['POST'])
@permission_classes((AllowAny,))
def login_and_get_token(request):
	"""
	User login (with authentication token)


	__Inputs__:

		- username (string, required): user's username
		- password (string, required): user's password

	__Output__:

		- error (json-dict): describes the error (if known) with a code (integer) and a message (string).
		- ret (json-dict): 
			- token (string): user's authentication token

	__Notes__:

		- None

	"""
	keys = (
		("username", "username", ""),
		("password", "password", ""),
	)
	res, error = dispatcher.login(request, get_as_or_get_default(request.data, keys))
	if error:
		return JsonResponse({"error" : error.regroup()})

	elif res.get("error") or not res.get("ret"):
		return JsonResponse({"error" : res.get("error", {"code": -1, "message" : "an error occured"})})

	else:
		res, error = dispatcher.get_authtoken(request, get_as_or_get_default(request.data, (("username", "username", ""),)))

			
	if error:
		return JsonResponse({"error": error.regroup()})
	return JsonResponse(res)




@api_view(['POST'])
@permission_classes((AllowAny,))
def send_password_reset_link(request):
	"""
	Send password reset


	__Inputs__:

		- username (string, required): user's username

	__Output__:

		- error (json-dict): describes the error (if known) with a code (integer) and a message (string).
		- ret (boolean): true if password link/code was sent succesfully

	__Notes__:

		- None

	"""
	keys = (
		("username", "username", ""),
	)
	res, error = dispatcher.send_password_reset_link(request, get_as_or_get_default(request.data, keys))
	if error:
		return JsonResponse({"error": error.regroup()})
	return JsonResponse(res)




@api_view(['POST'])
@permission_classes((AllowAny,))
def set_password_reset(request):
	"""
	Set password reset


	__Inputs__:

		- username (string, required): user's username
		- password (string, required): user's new password
		- identity (string, optional): password reset's identity (for web app)
		- code (string, optional): password reset's identity (for mobile apps)


	__Output__:

		- error (json-dict): describes the error (if known) with a code (integer) and a message (string).
		- ret (boolean): true if password link/code was sent succesfully

	__Notes__:

		- None

	"""
	keys = (
		("username", "username", ""),
		("password", "password", ""),
		("identity", "identity", ""),
		("code", "code", ""),
	)
	res, error = dispatcher.set_password_reset(request, get_as_or_get_default(request.data, keys))
	if error:
		print(error.original)
		return JsonResponse({"error": error.regroup()})
	return JsonResponse(res)


from django.shortcuts import render, redirect
from django.core.urlresolvers import reverse
from django.http import HttpResponse
from django.contrib.auth.decorators import login_required


def index(request):
	return HttpResponse("accounts:index")
	


from ohm2_handlers_light import utils as h_utils
from ohm2_accounts_light.decorators import ohm2_accounts_light_safe_request
from ohm2_accounts_light import utils as ohm2_accounts_light_utils
from ohm2_accounts_light import settings
from . import errors




@ohm2_accounts_light_safe_request
def signup(request, params):
	p = h_utils.cleaned(params,
						(
							("string", "username", 1),
							("string", "password", 1),
							("string", "email", 0),
						))

	if request.user.is_authenticated():
		return {"error": errors.USER_ALREADY_LOGGED_IN}
	
	elif ohm2_accounts_light_utils.user_exist(username = p["username"]):
		return {"error" : errors.USER_ALREADY_EXIST}

	elif settings.CHECK_PASSWORD_SECURE and not ohm2_accounts_light_utils.is_password_secure(p["password"]):
		return {"error" : errors.THE_PASSWORD_IS_NOT_SECURE}

	elif len(p["email"]) > 0 and not h_utils.is_email_valid(p["email"]):
		return {"error" : errors.INVALID_EMAIL}

	elif not settings.SIGNUPS_ENABLED:
		return {"error" : errors.SIGNUPS_DISABLED}

	else:
		pass


	username, password, email = p["username"], p["password"], p["email"]

	if len(email) == 0 and h_utils.is_email_valid(username):
		email = username

	
	user = ohm2_accounts_light_utils.create_user(username, email, password)

	res = {
		"error" : None,
		"ret" : True,
	}
	return res



@ohm2_accounts_light_safe_request
def login(request, params):
	p = h_utils.cleaned(params,
						(
							("string", "username", 1),
							("string", "password", 1),
						))


	if request.user.is_authenticated():
		return {"error": None, "ret" : True}
	
	else:
		username, password = p["username"], p["password"]

		
	if not settings.ENABLE_EMAIL_LOGIN and h_utils.is_email_valid(username):
		return {"error" : errors.EMAIL_LOGIN_DISABLED}

	elif settings.ENABLE_EMAIL_LOGIN and h_utils.is_email_valid(username) and ohm2_accounts_light_utils.user_exist(email = username) and settings.UNIQUE_USER_EMAILS:
		user = ohm2_accounts_light_utils.get_user(email = username)
		auth_user = ohm2_accounts_light_utils.user_authenticate(user.get_username(), password)
	
	else:

		auth_user = ohm2_accounts_light_utils.user_authenticate(username, password)
		
	
	
	if auth_user is None:
		return {"error" : errors.WRONG_CREDENTIALS}

	else:
		ohm2_accounts_light_utils.run_login_pipeline(request, auth_user)	
	

	res = {
		"error" : None,
		"ret" : True,
	}
	return res



@ohm2_accounts_light_safe_request
def logout(request, params):
	p = h_utils.cleaned(params,
						(
						))


	if not request.user.is_authenticated():
		return {"error": None, "ret" : False}
	
	else:
		ohm2_accounts_light_utils.run_logout_pipeline(request)	
	

	res = {
		"error" : None,
		"ret" : True,
	}
	return res


@ohm2_accounts_light_safe_request
def create_authtoken(request, params):
	p = h_utils.cleaned(params,
						(
							("string", "username", 1),
						))


	user = ohm2_accounts_light_utils.get_user(username = p["username"])
	token = ohm2_accounts_light_utils.get_or_create_authtoken(user)

	res = {
		"error" : None,
		"ret" : {
			"token" : token.key,
		}
	}
	return res




@ohm2_accounts_light_safe_request
def get_authtoken(request, params):
	p = h_utils.cleaned(params,
						(
							("string", "username", 1),
						))


	user = ohm2_accounts_light_utils.get_user(username = p["username"])
	token = ohm2_accounts_light_utils.get_or_create_authtoken(user)

	res = {
		"error" : None,
		"ret" : {
			"token" : token.key,
		}
	}
	return res



@ohm2_accounts_light_safe_request
def send_password_reset_link(request, params):
	p = h_utils.cleaned(params,
						(
							("string", "username", 1),
						))


	if request.user.is_authenticated():
		return {"error": None, "ret" : False}

	username = p["username"]

	if h_utils.is_email_valid(username) and settings.UNIQUE_USER_EMAILS:
		user = ohm2_accounts_light_utils.get_user_or_none(email = username)

	else:	
		user = ohm2_accounts_light_utils.get_or_none_user(username = username)
	
	if user is None:
		return {"error": None, "ret" : False}
		
	passwordreset = ohm2_accounts_light_utils.get_or_none_passwordreset(user = user, activation_date = None)
	if passwordreset is None:
		passwordreset = ohm2_accounts_light_utils.create_passwordreset(user)

	
	sent = False
	if passwordreset.send_again() and settings.PASSWORD_RESET_SEND_ENABLE:

		passwordreset, sent = ohm2_accounts_light_utils.send_passwordreset(passwordreset, request)
		


	res = {
		"error" : None,
		"ret" : sent,
	}
	return res









@ohm2_accounts_light_safe_request
def set_password_reset(request, params):
	p = h_utils.cleaned(params,
						(
							("string", "username", 1),
							("string", "password", 1),
							("string", "identity", 0),
							("string", "code", 0),
						))


	if request.user.is_authenticated():
		return {"error": None, "ret" : False}

	username, identity, code, password = p["username"], p["identity"], p["code"], p["password"]

	lookup = {}

	if len(identity) > 0:
		lookup["identity"] = identity

	elif len(code) > 0:
		lookup["code__iexact"] = code
	
	else:
		return {"error" : errors.PASSWORD_RESET_IDENTITY_AND_CODE_CANT_BE_BOTH_EMPTY}


	if h_utils.is_email_valid(username) and settings.UNIQUE_USER_EMAILS:
		lookup["user__email"] = username

	else:
		lookup["user__username"] = username
	

	passwordreset = ohm2_accounts_light_utils.get_or_none_passwordreset(**lookup)
	if passwordreset is None:
		return {"error" : errors.PASSWORD_RESET_INVALID}
	
	elif passwordreset.last_sent_date == None:
		return {"error" : errors.PASSWORD_RESET_INVALID}

	elif passwordreset.activation_date != None:
		return {"error" : errors.PASSWORD_RESET_ALREADY_ACTIVATED}

	elif settings.CHECK_PASSWORD_SECURE and not ohm2_accounts_light_utils.is_password_secure(password):
		return {"error" : errors.THE_PASSWORD_IS_NOT_SECURE}

	else:
		passwordreset = ohm2_accounts_light_utils.change_password_with_passwordreset(passwordreset, password)

		
	res = {
		"error" : None,
		"ret" : True,
	}
	return res





from django.core.exceptions import ObjectDoesNotExist, ValidationError
from django.contrib.auth.models import User
from django.contrib.auth import authenticate, login, logout
from django.contrib.auth.password_validation import validate_password, password_changed, password_validators_help_texts
from django.db.models import Q
from django.utils import timezone
from rest_framework.authtoken.models import Token
from ohm2_handlers_light import utils as h_utils
from ohm2_accounts_light import settings
from ohm2_accounts_light import models as ohm2_accounts_light_models
from ohm2_handlers_light import errors as ohm2_accounts_light_errors
from ohm2_handlers_light.definitions import RunException
from random import choice as random_choice
import importlib


def create_user(username, email, password, **kwargs):

	user = User.objects.create_user(username = username,
								    email = email,
								    password = password)
	

	return user

def get_user(**kwargs):
	return h_utils.db_get(User, **kwargs)

def get_or_none_user(**kwargs):
	return h_utils.db_get_or_none(User, **kwargs)

def filter_user(**kwargs):
	return h_utils.db_filter(obj = User, **kwargs)

def get_or_create_authtoken(user):
	token, created = Token.objects.get_or_create(user = user)
	return token

def get_authtoken(user):
	return Token.objects.get(user = user)

def user_exist(**kwargs):
	if filter_user(**kwargs).count() > 0:
		return True
	return False

def change_password(user, password):
	user.set_password(password)
	user.save()
	return user

def user_authenticate(username, password):
	return authenticate(username = username, password = password)

def user_login(request, auth_user):
	return login(request, auth_user)

def user_logout(request):
	return logout(request)

def validate_current_password(password, user = None, password_validators = None):
	try:
		validate_password(password, user, password_validators)
	except ValidationError as e:
		return [reason for reason in e]
	return []

def is_password_secure(password, user = None, password_validators = None):
	errors = validate_current_password(password, user, password_validators)
	if len(errors) == 0:
		return True
	return False

def run_login_pipeline(request, auth_user, **kwargs):
	previous_outputs = {}
	for pipeline in settings.LOGIN_PIPELINE:

		func = pipeline.rsplit(".", 1)[-1]
		
		m = importlib.import_module(pipeline)
		function = getattr(m, func)

		auth_user, output = function(request, auth_user, previous_outputs)

		previous_outputs = h_utils.join_dicts(previous_outputs, output)

	return None

def run_logout_pipeline(request, **kwargs):
	previous_outputs = {}
	for pipeline in settings.LOGOUT_PIPELINE:

		func = pipeline.rsplit(".", 1)[-1]
		
		m = importlib.import_module(pipeline)
		function = getattr(m, func)

		output = function(request, previous_outputs)

		previous_outputs = h_utils.join_dicts(previous_outputs, output)

	return None


def create_passwordreset(user, **kwargs):
	kwargs["user"] = user
	kwargs["code"] = get_passwordreset_code(5)
	return h_utils.db_create(ohm2_accounts_light_models.PasswordReset, **kwargs)

def get_passwordreset_code(length, **kwargs):
	possible_characters = "abcdefghkmnprstuvxyz123456789"
	
	max_tries, tries = 10, 0
	while tries < max_tries:
		tries += 1
		code = "".join(random_choice(possible_characters) for x in range(length))
		if get_or_none_passwordreset(code = code) is None:
			return code

	else:
		raise RunException(**ohm2_accounts_light_errors.NO_PASSWORD_RESET_FOUND)	


def get_passwordreset(**kwargs):
	return h_utils.db_get(ohm2_accounts_light_models.PasswordReset, **kwargs)

def get_or_none_passwordreset(**kwargs):
	return h_utils.db_get_or_none(ohm2_accounts_light_models.PasswordReset, **kwargs)

def send_passwordreset(passwordreset, request, **kwargs):
	context = {
		"ret" : {
			"passwordreset" : passwordreset,
			"website_url" : settings.WEBSITE_URL,
		},	
	}

	content = h_utils.template_response(settings.PASSWORD_RESET_TEMPLATE_PATH, context)
	

	sent = h_utils.mailgun_send(passwordreset.user.email, settings.PASSWORD_RESET_FROM_EMAIL, settings.PASSWORD_RESET_SUBJECT, content)


	if sent:
		passwordreset = h_utils.db_update(passwordreset, last_sent_date = timezone.now())
	
	return (passwordreset, sent)


def change_password_with_passwordreset(passwordreset, password, **kwargs):
	user = change_password(passwordreset.user, password)
	return h_utils.db_update(passwordreset, activation_date = timezone.now())






